////////////////////////////////////
//	Autor: Jakub Jdrzejczak
// 	Data: 16.04.2009
//	Tytu: DigiSpeed
//
//	Zasada dziaania:
//
//		Wyjscia:
//			- PWM na wyprowadzeniu OC1A - licznik T1 w trybie 10bit fast PWM
//			- wyprowadzenia PB0 i PB1 steruja przekaznikiami, wlaczane zaleznie
//			  od sygnalow na wejsciach M03 i M04 
//
//		Wejscia:
//			- STEP na wyprowadzeniu PD3 (INT1) powoduje zwiekszenie lub zmniejszenie wartosci licznika o 1 w momencie
//		      wystapienia zbocza opadajacego
//			- DIR na wyprowadzeniu PD4, stan niski powoduje zwiekszanie wartosci licznika w przypadku sygnalu STEP
//			  stan wysoki powoduje zmniejszanie wartosci licznika w przypadku wystapienia sygnalu STEP
//			- M03 na wyprowadzeniu PD5, stan wysoki powoduje uruchomienie wyjscia i pojawienie sie sygnalu wysokiego na wyjsciu przekaznika
//			- M03 i M04 ( PD6 ) j/w tylko drugie wyjscie
//
//			
//
//		Licznik:
//			Rejestry R20 i R21, maksymalna wartosc wynosi 1023 0x3FF minimalna 0, w przypadku nieaktywnego wyjscia wartosc licznika po sygnale Step
//			nie jest zmieniana.
///////////////////////////////////

.nolist
.include "tn2313def.inc"
.list


.def		mpr1 = r16

.def		counterLow = r20
.def		counterHigh = r21
.def		enabled = r22

.cseg


.org	0x0000
				rjmp	start
.org	INT0addr				; External Interrupt Request 0
				reti
.org	INT1addr				; External Interrupt Request 1
				rjmp	step
.org	ICP1addr				; Timer/Counter1 Capture Event
				reti
.org	OC1Aaddr				; Timer/Counter1 Compare Match A
				reti
.org	OVF1addr				; Timer/Counter1 Overflow
				reti
.org	OVF0addr				; Timer/Counter0 Overflow
				reti
.org	URXCaddr				; USART, Rx Complete
				reti
.org	UDREaddr				; USART Data Register Empty
				reti
.org	UTXCaddr				; USART, Tx Complete
				reti
.org	ACIaddr					; Analog Comparator
				reti
.org	PCIaddr					; 
				reti
.org	OC1Baddr				; 
				reti
.org	OC0Aaddr				; 
				reti
.org	OC0Baddr				; 
				reti
.org	USI_STARTaddr			; USI Start Condition
				reti
.org	USI_OVFaddr				; USI Overflow
				reti
.org	ERDYaddr				; 
				reti
.org	WDTaddr					; Watchdog Timer Overflow
				reti


Step:
		push	mpr1			; na stos
		in		mpr1,SREG
		push	mpr1

//		tst		enabled			; modyfikowanie licznika niezaleznie od enable
//		breq	StepEnd

		sbis	pind,pd4
		rjmp	StepAdd
		subi	counterLow,1
		sbci	counterHigh,0
		brcc	StepEnd
		clr		counterLow
		clr		counterHigh
		rjmp	StepEnd

StepAdd:
		ldi		mpr1,1
		add		counterLow,mpr1
		ldi		mpr1,0
		adc		counterHigh,mpr1
		cpi		counterLow,1023%256
		ldi		mpr1,1023/256
		cpc		counterHigh,mpr1
		brlo	StepEnd
		ldi		counterLow,1023%256
		ldi		counterHigh,1023/256

StepEnd:
		pop		mpr1			; ze stosu
		out		SREG,mpr1
		pop		mpr1
		reti


// ***************

Start:

		ldi		mpr1,low(RAMEND)		// wskaznik stosu na koniec ramu
		out		SPL,mpr1
		
		clr		counterLow				
		clr		counterHigh

		cbi		portb,pb0				// wyjscia przekaznikowe
		cbi		portb,pb1
		sbi		ddrb,pb1
		sbi		ddrb,pb0

		ldi		mpr1,(1<<COM1A1)|(1<<WGM10)|(1<<WGM11)	// Timer 1 FAST PWM na wyjsciu OC1A
		out		TCCR1A,mpr1
		ldi		mpr1,(1<<WGM12)|(1<<CS10)
		out		TCCR1B,mpr1

		ldi		mpr1,(1<<ISC11)|(0<<ISC10)				// INT1 wyzwalane zboczem narastajacym
		out		MCUCR,mpr1
		ldi		mpr1,(1<<INT1)
		out		GIMSK,mpr1

		sei								// globalnie przerwania wlaczone

Loop:
		out		OCR1AH,counterHigh		// wartosci z licznika do rejestrow odpowiedzialnych za wyjscie OC1A
		out		OCR1AL,counterLow		

		sbic	pind,pd5				// a tu juz sprawdzanie dla M03 i M04
		rjmp	LoopM03High
//		sbic	pind,pd6
//		rjmp	LoopM04High
LoopDisabled:							
		clr		enabled
		cbi		portb,pb0
		cbi		portb,pb1
		cbi		ddrb,pb3
		rjmp	Loop

LoopM03High:
		sbic	pind,pd6
		rjmp	LoopM04High
		ldi		enabled,3
		sbi		portb,pb1
		cbi		portb,pb0
		sbi		ddrb,pb3
		rjmp	Loop

LoopM04High:
		ldi		enabled,4
		sbi		portb,pb0
		cbi		portb,pb1
		sbi		ddrb,pb3
		rjmp	Loop
